package co.ab180.airbridge.flutter

import co.ab180.airbridge.Airbridge
import co.ab180.airbridge.event.StandardEventCategory
import co.ab180.airbridge.flutter.utility.set
import co.ab180.airbridge.flutter.utility.trackEvent
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

class EventAPI(private val channel: MethodChannel) : MethodChannel.MethodCallHandler {
    companion object

    fun attachToChannel() {
        channel.setMethodCallHandler(this)
    }

    fun detachFromChannel() {
        channel.setMethodCallHandler(null)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        val arguments = call.arguments as? Map<*, *> ?: HashMap<Any, Any>()

        when (call.method) {
            "sendSignIn" -> sendSignIn(arguments, result)
            "sendSignUp" -> sendSignUp(arguments, result)
            "sendSignOut" -> sendSignOut(arguments, result)
            "sendViewHome" -> sendStandardEvent(StandardEventCategory.HOME_VIEW, arguments, result)
            "sendViewSearchResult" -> sendStandardEvent(StandardEventCategory.SEARCH_RESULT_VIEW, arguments, result)
            "sendViewProductList" -> sendStandardEvent(StandardEventCategory.PRODUCT_LIST_VIEW, arguments, result)
            "sendViewProductDetail" -> sendStandardEvent(StandardEventCategory.PRODUCT_DETAILS_VIEW, arguments, result)
            "sendAddToCart" -> sendStandardEvent(StandardEventCategory.ADD_TO_CART, arguments, result)
            "sendPurchase" -> sendStandardEvent(StandardEventCategory.ORDER_COMPLETED, arguments, result)
            "sendCustom" -> sendCustomEvent(arguments, result)
            else -> result.notImplemented()
        }
    }

    private fun sendSignIn(arguments: Map<*, *>, result: MethodChannel.Result) {
        Airbridge.getCurrentUser().set(user = arguments["user"] as? Map<*, *>)
        Airbridge.trackEvent(category = StandardEventCategory.SIGN_IN, option = arguments["option"] as? Map<*, *>)

        result.success(null)
    }

    private fun sendSignUp(arguments: Map<*, *>, result: MethodChannel.Result) {
        Airbridge.getCurrentUser().set(user = arguments["user"] as? Map<*, *>)
        Airbridge.trackEvent(category = StandardEventCategory.SIGN_UP, option = arguments["option"] as? Map<*, *>)

        result.success(null)
    }

    private fun sendSignOut(arguments: Map<*, *>, result: MethodChannel.Result) {
        Airbridge.trackEvent(category = StandardEventCategory.SIGN_OUT, option = arguments["option"] as? Map<*, *>)
        Airbridge.getCurrentUser().set(user = null)

        result.success(null)
    }

    private fun sendStandardEvent(category: StandardEventCategory, arguments: Map<*, *>, result: MethodChannel.Result) {
        sendEvent(category.category, arguments, result)
    }

    private fun sendCustomEvent(arguments: Map<*, *>, result: MethodChannel.Result) {
        (arguments["category"] as? String)?.let { category ->
            sendEvent(category, arguments, result)
        }
    }

    private fun sendEvent(category: String, arguments: Map<*, *>, result: MethodChannel.Result) {
        Airbridge.trackEvent(category = category, option = arguments["option"] as? Map<*, *>)

        result.success(null)
    }
}

fun EventAPI.Companion.fromPlugin(binding: FlutterPlugin.FlutterPluginBinding): EventAPI {
    return EventAPI(MethodChannel(binding.binaryMessenger, "airbridge_flutter_sdk/method/event"))
}