package co.ab180.airbridge.flutter

import co.ab180.airbridge.Airbridge
import co.ab180.airbridge.AirbridgeCallback
import co.ab180.airbridge.flutter.utility.set
import co.ab180.airbridge.flutter.utility.update
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

class StateAPI(private val channel: MethodChannel) : MethodChannel.MethodCallHandler {
    companion object

    fun attachToChannel() {
        channel.setMethodCallHandler(this)
    }

    fun detachFromChannel() {
        channel.setMethodCallHandler(null)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        val arguments = call.arguments as? Map<*, *> ?: HashMap<Any, Any>()

        when (call.method) {
            "setUser" -> setUser(arguments, result)
            "updateUser" -> updateUser(arguments, result)
            "startTracking" -> startTracking(arguments, result)
            "getDeviceUUID" -> getDeviceUUID(arguments, result)
            else -> result.notImplemented()
        }
    }

    private fun setUser(arguments: Map<*, *>, result: MethodChannel.Result) {
        Airbridge.getCurrentUser().set(user = arguments)

        result.success(null)
    }

    private fun updateUser(arguments: Map<*, *>, result: MethodChannel.Result) {
        Airbridge.getCurrentUser().update(user = arguments)

        result.success(null)
    }

    private fun startTracking(arguments: Map<*, *>, result: MethodChannel.Result) {
        Airbridge.startTracking()

        result.success(null)
    }

    private fun getDeviceUUID(arguments: Map<*, *>, result: MethodChannel.Result) {
        val channelResult = result

        Airbridge.getDeviceInfo().getUUID(object: AirbridgeCallback<String> {
            override fun onSuccess(result: String) {
                channelResult.success(result)
            }

            override fun onFailure(throwable: Throwable) {
                channelResult.success(null)
            }

            override fun onComplete() {}
        })
    }
}

fun StateAPI.Companion.fromPlugin(binding: FlutterPlugin.FlutterPluginBinding): StateAPI {
    return StateAPI(MethodChannel(binding.binaryMessenger, "airbridge_flutter_sdk/method/state"))
}