//
//  StateAPI.swift
//  airbridge_flutter_sdk
//
//  Created by WOF on 2021/07/05.
//

import Foundation
import AirBridge

struct StateAPI {
    let channel: FlutterMethodChannel
}

extension StateAPI {
    mutating func attachToChannel() {
        channel.setMethodCallHandler(handle(call:result:))
    }
    
    mutating func detachFromChannel() {
        channel.setMethodCallHandler(nil)
    }
}

extension StateAPI {
    func handle(call: FlutterMethodCall, result: FlutterResult) {
        let arguments = call.arguments as? [AnyHashable: Any] ?? [:]
        
        switch call.method {
        case "setUser": setUser(arguments, result)
        case "updateUser": updateUser(arguments, result)
        case "startTracking": startTracking(arguments, result)
        case "getDeviceUUID": getDeviceUUID(arguments, result)
        default: result(FlutterMethodNotImplemented)
        }
    }
}

extension StateAPI {
    func setUser(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        AirBridge.state().setUser(from: arguments)
        
        result(nil)
    }
    
    func updateUser(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        AirBridge.state().updateUser(from: arguments)
        
        result(nil)
    }
    
    func startTracking(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        AirBridge.startTracking()
        
        result(nil)
    }
    
    func getDeviceUUID(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        result(AirBridge.deviceUUID())
    }
}

extension StateAPI {
    static func fromPlugin(_ registrar: FlutterPluginRegistrar) -> StateAPI {
        return StateAPI(channel: FlutterMethodChannel(name: "airbridge_flutter_sdk/method/state",
                                                      binaryMessenger: registrar.messenger()))
    }
}

